require 'tmpdir'
require 'tempfile'
require 'fileutils'
require 'minitest/autorun'

require_relative '../lib/md2dw'


class TestMD2DW < MiniTest::Test

  def test_converts_markdown_to_dokuwiki
    [ # Markdown => DokuWiki

      # Text
      [ '*EM*', '//EM//' ],
      [ '**BOLD**', '**BOLD**' ],
      [ '<sup>SUP</sup>', '<sup>SUP</sup>' ],
      [ '`MONO`', "''MONO''" ],
      [ '~~STRIKE~~', '<del>STRIKE</del>' ],

      # Links
      [ '[LINK](THIS)', '[[THIS|LINK]]' ],

      # Headings
      [ '# TEST', '====== TEST ======' ],
      [ '## TEST', '===== TEST =====' ],
      [ '### TEST', '==== TEST ====' ],
      [ '#### TEST', '=== TEST ===' ],
      [ '##### TEST', '== TEST ==' ],
      [ '###### TEST', '= TEST =' ],

      # Images
      [ '![ALT](IMG.jpg)', '{{IMG.jpg}}' ],

      # Lists
      [ ul_list_md, ul_list_dw ],
      [ ol_list_md, ol_list_dw ],

      # Quotes
      [ '<cite>CITE</cite>', '<q>CITE</q>' ],
      [ blockquote_md, blockquote_dw ],

      # Tables
      [ table_md1, table_dw ],
      [ table_md2, table_dw ],

      # Code
      [ code_block_md, code_block_dw ],
      [ code_block_mde_md, code_block_mde_dw ],
      [ code_block_gfm_md, code_block_gfm_dw ],

      # Special edge-case for double-dashes in code samples
      [ "`--TEST`", "''%%--TEST%%''" ]

    ].map { |tag| assert_renders tag[0], tag[1] }
  end

private
  def assert_renders md, dw
    assert_equal dw, MD2DW.render(md)
  end

  def ul_list_md
    <<-EOL.gsub(/^ {4}/, '').chomp
    TEST

    - ONE
    - TWO
    EOL
  end

  def ul_list_dw
    <<-EOL.gsub(/^ {4}/, '').chomp
    TEST

      * ONE
      * TWO
    EOL
  end

  def ol_list_md
    <<-EOL.gsub(/^ {4}/, '').chomp
    TEST

    1. ONE
    2. TWO
    EOL
  end

  def ol_list_dw
    <<-EOL.gsub(/^ {4}/, '').chomp
    TEST

      - ONE
      - TWO
    EOL
  end

  def blockquote_md
    <<-EOB.gsub(/^ {4}/, '').chomp
    > ONE
    > TWO
    > THREE
    EOB
  end

  def blockquote_dw
    <<-EOB.gsub(/^ {4}/, '').chomp
    <blockquote>
    ONE
    TWO
    THREE
    </blockquote>
    EOB
  end

  def table_md1
    <<-EOT.gsub(/^ {4}/, '')
    First Header  | Second Header
    ------------- | -------------
    Content Cell  | Content Cell
    Content Cell  | Content Cell
    EOT
  end

  def table_md2
    <<-EOT.gsub(/^ {4}/, '')
    | First Header  | Second Header |
    | ------------- | ------------- |
    | Content Cell  | Content Cell  |
    | Content Cell  | Content Cell  |
    EOT
  end

  def table_dw
    <<-EOT.gsub(/^ {4}/, '').chomp
    ^ First Header ^ Second Header ^
    | Content Cell | Content Cell |
    | Content Cell | Content Cell |
    EOT
  end

  def code_block_md
    <<-EOC.gsub(/^ {4}/, '')
    TEST

        ONE
        TWO
    EOC
  end

  def code_block_dw
    <<-EOC.gsub(/^ {4}/, '').chomp
    TEST

    <code>
    ONE
    TWO
    </code>
    EOC
  end

  def code_block_mde_md
    <<-EOC.gsub(/^ {4}/, '')
    TEST

    ~~~
    ONE
    TWO
    ~~~
    EOC
  end

  def code_block_mde_dw
    <<-EOC.gsub(/^ {4}/, '').chomp
    TEST

    <code>
    ONE
    TWO
    </code>
    EOC
  end

  def code_block_gfm_md
    <<-EOC.gsub(/^ {4}/, '')
    TEST

    ```ruby
    ONE
    TWO
    ```
    EOC
  end

  def code_block_gfm_dw
    <<-EOC.gsub(/^ {4}/, '').chomp
    TEST

    <code ruby>
    ONE
    TWO
    </code>
    EOC
  end
end